unit LogBookUnit;

interface
 uses Generics.Collections, Generics.Defaults,SpaceObjectUnit, SysUtils;
 type
  TLogBook=class(TObject)
    private
      SourceList: TList<TSpaceObject>;
      fCatalog:string;
    public
     List:TList<TSpaceObject>;
     constructor Create;
     property Catalog:string read fCatalog;
     procedure LoadFromFile(FileName:string);
     procedure Print;
     procedure SortByName;
     procedure SortByDiameter;
     procedure Filter(MinDiameter:integer=0;MaxDiameter:integer=0);
     destructor Destroy;
  end;

implementation

  constructor TLogBook.Create;
  begin
    SourceList:=TList<TSpaceObject>.Create;
    List:=TList<TSpaceObject>.Create;
  end;

  procedure TLogBook.LoadFromFile(FileName: string);
  var
    f:TextFile;
    objType:string;
    obj:TSpaceObject;
  begin
    fCatalog:=ExtractFileDir(FileName);
    SourceList.Clear;
    AssignFile(f,fileName);
    Reset(f);
    while not eof(f) do
    begin
      readln(f,objType);
      if objType='' then Continue;
      if objType='' then
        obj:=TArtificialObject.Create
      else
        obj:=TNaturalObject.Create;
      obj.ReadFromFile(f);
      SourceList.Add(obj);
    end;
    CloseFile(f);
    Filter;
  end;

  procedure TLogBook.Print;
  var
    i: Integer;
  begin
    writeln(' :');
    for i := 0 to List.Count - 1 do
    begin
      writeln;
      list[i].Print;
    end;
  end;

  destructor TLogBook.Destroy;
  var i:integer;
  begin
    for i := 0 to SourceList.Count - 1 do
      SourceList[i].Free;
    SourceList.Free;
    List.Free;
  end;

  procedure TLogBook.SortByName;
  begin
   if List.Count>0 then
    List.Sort(TComparer<TSpaceObject>.Construct(
      function (const Left,Right:TSpaceObject):integer
      begin
        result:=1;
        if(Left.Name<Right.Name) then result:=-1
        else if (Left.Name=Right.Name) then result:=0;
      end
    ));
  end;

  procedure TLogBook.SortByDiameter;
  begin
   if List.Count>0 then
    List.Sort(TComparer<TSpaceObject>.Construct(
      function (const Left,Right:TSpaceObject):integer
      begin
        result:=Left.Diameter-Right.Diameter;
      end
    ));
  end;

  procedure TLogBook.Filter(MinDiameter: Integer = 0; MaxDiameter: Integer = 0);
  var i:integer;
  begin
    List.Clear; // 
    for i := 0 to SourceList.Count - 1 do
      if (MinDiameter<=0) or (SourceList[i].Diameter>=MinDiameter) then
        if (MaxDiameter<=0) or (SourceList[i].Diameter<=MaxDiameter) then
          List.Add(sourceList[i]);
  end;

end.
